@@ -26,10 +26,11 @@ import testtools.matchers as ttm
 from heat_cfntools.cfntools import cfn_helper
 
 
-def popen_root_calls(calls):
-    kwargs = {'env': None, 'cwd': None, 'stderr': -1, 'stdout': -1}
+def popen_root_calls(calls, shell=False):
+    kwargs = {'env': None, 'cwd': None, 'stderr': -1, 'stdout': -1,
+              'shell': shell}
     return [
-        mock.call(call.split(), **kwargs)
+        mock.call(call, **kwargs)
         for call in calls
     ]
 
@@ -47,9 +48,12 @@ class FakePOpen():
         pass
 
 
+@mock.patch.object(cfn_helper.pwd, 'getpwnam')
+@mock.patch.object(cfn_helper.os, 'seteuid')
+@mock.patch.object(cfn_helper.os, 'geteuid')
 class TestCommandRunner(testtools.TestCase):
 
-    def test_command_runner(self):
+    def test_command_runner(self, mock_geteuid, mock_seteuid, mock_getpwnam):
         def returns(*args, **kwargs):
             if args[0][0] == '/bin/command1':
                 return FakePOpen('All good')
@@ -60,74 +64,88 @@ class TestCommandRunner(testtools.TestCase):
 
         with mock.patch('subprocess.Popen') as mock_popen:
             mock_popen.side_effect = returns
-            cmd2 = cfn_helper.CommandRunner('/bin/command2')
-            cmd1 = cfn_helper.CommandRunner('/bin/command1', cmd2)
+            cmd2 = cfn_helper.CommandRunner(['/bin/command2'])
+            cmd1 = cfn_helper.CommandRunner(['/bin/command1'],
+                                            nextcommand=cmd2)
             cmd1.run('root')
             self.assertEqual(
-                'CommandRunner:\n\tcommand: /bin/command1\n\tstdout: All good',
+                'CommandRunner:\n\tcommand: [\'/bin/command1\']\n\tstdout: '
+                'All good',
                 str(cmd1))
             self.assertEqual(
-                'CommandRunner:\n\tcommand: /bin/command2\n\tstatus: -1\n'
-                '\tstdout: Doing something\n\tstderr: error',
+                'CommandRunner:\n\tcommand: [\'/bin/command2\']\n\tstatus: '
+                '-1\n\tstdout: Doing something\n\tstderr: error',
                 str(cmd2))
-            calls = popen_root_calls(['/bin/command1', '/bin/command2'])
+            calls = popen_root_calls([['/bin/command1'], ['/bin/command2']])
             mock_popen.assert_has_calls(calls)
 
-    def test_runner_runs_command_as_list(self):
-        with mock.patch('subprocess.Popen') as mock_popen:
-            command = '/bin/command --option=value arg1 arg2'
-            expected_cmd = ['/bin/command', '--option=value', 'arg1', 'arg2']
-            cmd = cfn_helper.CommandRunner(command)
-            cmd.run()
-            self.assertTrue(mock_popen.called)
-            actual_cmd = mock_popen.call_args[0][0]
-            self.assertTrue(isinstance(actual_cmd, list))
-            self.assertItemsEqual(expected_cmd, actual_cmd)
-
-    @mock.patch.object(cfn_helper.pwd, 'getpwnam')
-    @mock.patch.object(cfn_helper.os, 'setuid')
-    def test_privileges_are_lowered_for_non_root_user(self, mock_setuid,
+    def test_privileges_are_lowered_for_non_root_user(self, mock_geteuid,
+                                                      mock_seteuid,
                                                       mock_getpwnam):
         pw_entry = mock.MagicMock()
         pw_entry.pw_uid = 1001
         mock_getpwnam.return_value = pw_entry
+        mock_geteuid.return_value = 0
+        calls = [mock.call(1001), mock.call(0)]
         with mock.patch('subprocess.Popen') as mock_popen:
-            command = '/bin/command --option=value arg1 arg2'
+            command = ['/bin/command', '--option=value', 'arg1', 'arg2']
             cmd = cfn_helper.CommandRunner(command)
             cmd.run(user='nonroot')
+            self.assertTrue(mock_geteuid.called)
             mock_getpwnam.assert_called_once_with('nonroot')
-            mock_setuid.assert_called_once_with(1001)
+            mock_seteuid.assert_has_calls(calls)
             self.assertTrue(mock_popen.called)
 
-    @mock.patch.object(cfn_helper.pwd, 'getpwnam')
-    @mock.patch.object(cfn_helper.os, 'setuid')
-    def test_run_returns_when_cannot_set_privileges(self, mock_setuid,
+    def test_run_returns_when_cannot_set_privileges(self, mock_geteuid,
+                                                    mock_seteuid,
                                                     mock_getpwnam):
-        mock_setuid.side_effect = Exception('[Error 1] Permission Denied')
+        msg = '[Error 1] Permission Denied'
+        mock_seteuid.side_effect = Exception(msg)
         with mock.patch('subprocess.Popen') as mock_popen:
-            command = '/bin/command2'
+            command = ['/bin/command2']
             cmd = cfn_helper.CommandRunner(command)
             cmd.run(user='nonroot')
             self.assertTrue(mock_getpwnam.called)
-            self.assertTrue(mock_setuid.called)
+            self.assertTrue(mock_seteuid.called)
             self.assertFalse(mock_popen.called)
             self.assertEqual(126, cmd.status)
+            self.assertEqual(msg, cmd.stderr)
+
+    def test_privileges_are_restored_for_command_failure(self, mock_geteuid,
+                                                         mock_seteuid,
+                                                         mock_getpwnam):
+        pw_entry = mock.MagicMock()
+        pw_entry.pw_uid = 1001
+        mock_getpwnam.return_value = pw_entry
+        mock_geteuid.return_value = 0
+        calls = [mock.call(1001), mock.call(0)]
+        with mock.patch('subprocess.Popen') as mock_popen:
+            mock_popen.side_effect = ValueError('Something wrong')
+            command = ['/bin/command', '--option=value', 'arg1', 'arg2']
+            cmd = cfn_helper.CommandRunner(command)
+            self.assertRaises(ValueError, cmd.run, user='nonroot')
+            self.assertTrue(mock_geteuid.called)
+            mock_getpwnam.assert_called_once_with('nonroot')
+            mock_seteuid.assert_has_calls(calls)
+            self.assertTrue(mock_popen.called)
 
 
+@mock.patch.object(cfn_helper, 'controlled_privileges')
 class TestPackages(testtools.TestCase):
 
-    def test_yum_install(self):
+    def test_yum_install(self, mock_cp):
 
         def returns(*args, **kwargs):
-            if (args[0][0] == 'rpm' and args[0][1] == '-q'):
+            if args[0][0] == 'rpm' and args[0][1] == '-q':
                 return FakePOpen(returncode=1)
             else:
                 return FakePOpen(returncode=0)
 
-        calls = ['which yum']
+        calls = [['which', 'yum']]
         for pack in ('httpd', 'wordpress', 'mysql-server'):
-            calls.append('rpm -q %s' % pack)
-            calls.append('yum -y --showduplicates list available %s' % pack)
+            calls.append(['rpm', '-q', pack])
+            calls.append(['yum', '-y', '--showduplicates', 'list',
+                          'available', pack])
         calls = popen_root_calls(calls)
 
         packages = {
@@ -143,7 +161,7 @@ class TestPackages(testtools.TestCase):
             cfn_helper.PackagesHandler(packages).apply_packages()
             mock_popen.assert_has_calls(calls, any_order=True)
 
-    def test_dnf_install_yum_unavailable(self):
+    def test_dnf_install_yum_unavailable(self, mock_cp):
 
         def returns(*args, **kwargs):
             if ((args[0][0] == 'rpm' and args[0][1] == '-q')
@@ -152,10 +170,11 @@ class TestPackages(testtools.TestCase):
             else:
                 return FakePOpen(returncode=0)
 
-        calls = ['which yum']
+        calls = [['which', 'yum']]
         for pack in ('httpd', 'wordpress', 'mysql-server'):
-            calls.append('rpm -q %s' % pack)
-            calls.append('dnf -y --showduplicates list available %s' % pack)
+            calls.append(['rpm', '-q', pack])
+            calls.append(['dnf', '-y', '--showduplicates', 'list',
+                          'available', pack])
         calls = popen_root_calls(calls)
 
         packages = {
@@ -171,18 +190,19 @@ class TestPackages(testtools.TestCase):
             cfn_helper.PackagesHandler(packages).apply_packages()
             mock_popen.assert_has_calls(calls, any_order=True)
 
-    def test_dnf_install(self):
+    def test_dnf_install(self, mock_cp):
 
         def returns(*args, **kwargs):
-            if (args[0][0] == 'rpm' and args[0][1] == '-q'):
+            if args[0][0] == 'rpm' and args[0][1] == '-q':
                 return FakePOpen(returncode=1)
             else:
                 return FakePOpen(returncode=0)
 
         calls = []
         for pack in ('httpd', 'wordpress', 'mysql-server'):
-            calls.append('rpm -q %s' % pack)
-            calls.append('dnf -y --showduplicates list available %s' % pack)
+            calls.append(['rpm', '-q', pack])
+            calls.append(['dnf', '-y', '--showduplicates', 'list',
+                          'available', pack])
         calls = popen_root_calls(calls)
 
         packages = {
@@ -198,18 +218,18 @@ class TestPackages(testtools.TestCase):
             cfn_helper.PackagesHandler(packages).apply_packages()
             mock_popen.assert_has_calls(calls, any_order=True)
 
-    def test_zypper_install(self):
+    def test_zypper_install(self, mock_cp):
 
         def returns(*args, **kwargs):
-            if (args[0][0] == 'rpm' and args[0][1] == '-q'):
+            if args[0][0].startswith('rpm') and args[0][1].startswith('-q'):
                 return FakePOpen(returncode=1)
             else:
                 return FakePOpen(returncode=0)
 
         calls = []
         for pack in ('httpd', 'wordpress', 'mysql-server'):
-            calls.append('rpm -q %s' % pack)
-            calls.append('zypper -n --no-refresh search %s' % pack)
+            calls.append(['rpm', '-q', pack])
+            calls.append(['zypper', '-n', '--no-refresh', 'search', pack])
         calls = popen_root_calls(calls)
 
         packages = {
@@ -225,7 +245,7 @@ class TestPackages(testtools.TestCase):
             cfn_helper.PackagesHandler(packages).apply_packages()
             mock_popen.assert_has_calls(calls, any_order=True)
 
-    def test_apt_install(self):
+    def test_apt_install(self, mock_cp):
         packages = {
             "apt": {
                 "mysql-server": [],
@@ -240,48 +260,58 @@ class TestPackages(testtools.TestCase):
             self.assertTrue(mock_popen.called)
 
 
+@mock.patch.object(cfn_helper, 'controlled_privileges')
 class TestServicesHandler(testtools.TestCase):
 
-    def test_services_handler_systemd(self):
+    def test_services_handler_systemd(self, mock_cp):
         calls = []
         returns = []
 
         # apply_services
-        calls.append('/bin/systemctl enable httpd.service')
+        calls.append(['/bin/systemctl', 'enable', 'httpd.service'])
         returns.append(FakePOpen())
-        calls.append('/bin/systemctl status httpd.service')
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start httpd.service')
+        calls.append(['/bin/systemctl', 'start', 'httpd.service'])
         returns.append(FakePOpen())
-        calls.append('/bin/systemctl enable mysqld.service')
+        calls.append(['/bin/systemctl', 'enable', 'mysqld.service'])
         returns.append(FakePOpen())
-        calls.append('/bin/systemctl status mysqld.service')
+        calls.append(['/bin/systemctl', 'status', 'mysqld.service'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start mysqld.service')
+        calls.append(['/bin/systemctl', 'start', 'mysqld.service'])
         returns.append(FakePOpen())
 
         # monitor_services not running
-        calls.append('/bin/systemctl status httpd.service')
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start httpd.service')
-        returns.append(FakePOpen())
-        calls.append('/bin/services_restarted')
-        returns.append(FakePOpen())
-        calls.append('/bin/systemctl status mysqld.service')
-        returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start mysqld.service')
-        returns.append(FakePOpen())
-        calls.append('/bin/services_restarted')
-        returns.append(FakePOpen())
-
-        # monitor_services running
-        calls.append('/bin/systemctl status httpd.service')
-        returns.append(FakePOpen())
-        calls.append('/bin/systemctl status mysqld.service')
+        calls.append(['/bin/systemctl', 'start', 'httpd.service'])
         returns.append(FakePOpen())
 
         calls = popen_root_calls(calls)
 
+        calls.extend(popen_root_calls(['/bin/services_restarted'], shell=True))
+        returns.append(FakePOpen())
+
+        calls.extend(popen_root_calls([['/bin/systemctl', 'status',
+                                        'mysqld.service']]))
+        returns.append(FakePOpen(returncode=-1))
+        calls.extend(popen_root_calls([['/bin/systemctl', 'start',
+                                        'mysqld.service']]))
+        returns.append(FakePOpen())
+
+        calls.extend(popen_root_calls(['/bin/services_restarted'], shell=True))
+        returns.append(FakePOpen())
+
+        # monitor_services running
+        calls.extend(popen_root_calls([['/bin/systemctl', 'status',
+                                        'httpd.service']]))
+        returns.append(FakePOpen())
+        calls.extend(popen_root_calls([['/bin/systemctl', 'status',
+                                        'mysqld.service']]))
+        returns.append(FakePOpen())
+
+        #calls = popen_root_calls(calls)
+
         services = {
             "systemd": {
                 "mysqld": {"enabled": "true", "ensureRunning": "true"},
@@ -312,16 +342,16 @@ class TestServicesHandler(testtools.TestCase):
                 mock_popen.assert_has_calls(calls, any_order=True)
             mock_exists.assert_called_with('/bin/systemctl')
 
-    def test_services_handler_systemd_disabled(self):
+    def test_services_handler_systemd_disabled(self, mock_cp):
         calls = []
 
         # apply_services
-        calls.append('/bin/systemctl disable httpd.service')
-        calls.append('/bin/systemctl status httpd.service')
-        calls.append('/bin/systemctl stop httpd.service')
-        calls.append('/bin/systemctl disable mysqld.service')
-        calls.append('/bin/systemctl status mysqld.service')
-        calls.append('/bin/systemctl stop mysqld.service')
+        calls.append(['/bin/systemctl', 'disable', 'httpd.service'])
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
+        calls.append(['/bin/systemctl', 'stop', 'httpd.service'])
+        calls.append(['/bin/systemctl', 'disable', 'mysqld.service'])
+        calls.append(['/bin/systemctl', 'status', 'mysqld.service'])
+        calls.append(['/bin/systemctl', 'stop', 'mysqld.service'])
         calls = popen_root_calls(calls)
 
         services = {
@@ -347,7 +377,7 @@ class TestServicesHandler(testtools.TestCase):
                 mock_popen.assert_has_calls(calls, any_order=True)
             mock_exists.assert_called_with('/bin/systemctl')
 
-    def test_services_handler_sysv_service_chkconfig(self):
+    def test_services_handler_sysv_service_chkconfig(self, mock_cp):
 
         def exists(*args, **kwargs):
             return args[0] != '/bin/systemctl'
@@ -356,27 +386,28 @@ class TestServicesHandler(testtools.TestCase):
         returns = []
 
         # apply_services
-        calls.append('/sbin/chkconfig httpd on')
+        calls.append(['/sbin/chkconfig', 'httpd', 'on'])
         returns.append(FakePOpen())
-        calls.append('/sbin/service httpd status')
+        calls.append(['/sbin/service', 'httpd', 'status'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/sbin/service httpd start')
+        calls.append(['/sbin/service', 'httpd', 'start'])
         returns.append(FakePOpen())
 
         # monitor_services not running
-        calls.append('/sbin/service httpd status')
+        calls.append(['/sbin/service', 'httpd', 'status'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/sbin/service httpd start')
-        returns.append(FakePOpen())
-        calls.append('/bin/services_restarted')
-        returns.append(FakePOpen())
-
-        # monitor_services running
-        calls.append('/sbin/service httpd status')
+        calls.append(['/sbin/service', 'httpd', 'start'])
         returns.append(FakePOpen())
 
         calls = popen_root_calls(calls)
 
+        calls.extend(popen_root_calls(['/bin/services_restarted'], shell=True))
+        returns.append(FakePOpen())
+
+        # monitor_services running
+        calls.extend(popen_root_calls([['/sbin/service', 'httpd', 'status']]))
+        returns.append(FakePOpen())
+
         services = {
             "sysvinit": {
                 "httpd": {"enabled": "true", "ensureRunning": "true"}
@@ -407,16 +438,16 @@ class TestServicesHandler(testtools.TestCase):
             mock_exists.assert_any_call('/sbin/service')
             mock_exists.assert_any_call('/sbin/chkconfig')
 
-    def test_services_handler_sysv_disabled_service_chkconfig(self):
+    def test_services_handler_sysv_disabled_service_chkconfig(self, mock_cp):
         def exists(*args, **kwargs):
             return args[0] != '/bin/systemctl'
 
         calls = []
 
         # apply_services
-        calls.append('/sbin/chkconfig httpd off')
-        calls.append('/sbin/service httpd status')
-        calls.append('/sbin/service httpd stop')
+        calls.append(['/sbin/chkconfig', 'httpd', 'off'])
+        calls.append(['/sbin/service', 'httpd', 'status'])
+        calls.append(['/sbin/service', 'httpd', 'stop'])
 
         calls = popen_root_calls(calls)
 
@@ -445,31 +476,35 @@ class TestServicesHandler(testtools.TestCase):
             mock_exists.assert_any_call('/sbin/service')
             mock_exists.assert_any_call('/sbin/chkconfig')
 
-    def test_services_handler_sysv_systemctl(self):
+    def test_services_handler_sysv_systemctl(self, mock_cp):
         calls = []
         returns = []
 
         # apply_services
-        calls.append('/bin/systemctl enable httpd.service')
+        calls.append(['/bin/systemctl', 'enable', 'httpd.service'])
         returns.append(FakePOpen())
-        calls.append('/bin/systemctl status httpd.service')
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start httpd.service')
+        calls.append(['/bin/systemctl', 'start', 'httpd.service'])
         returns.append(FakePOpen())
 
         # monitor_services not running
-        calls.append('/bin/systemctl status httpd.service')
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/bin/systemctl start httpd.service')
-        returns.append(FakePOpen())
-        calls.append('/bin/services_restarted')
+        calls.append(['/bin/systemctl', 'start', 'httpd.service'])
         returns.append(FakePOpen())
 
-        # monitor_services running
-        calls.append('/bin/systemctl status httpd.service')
+        shell_calls = []
+        shell_calls.append('/bin/services_restarted')
         returns.append(FakePOpen())
 
         calls = popen_root_calls(calls)
+        calls.extend(popen_root_calls(shell_calls, shell=True))
+
+        # monitor_services running
+        calls.extend(popen_root_calls([['/bin/systemctl', 'status',
+                                        'httpd.service']]))
+        returns.append(FakePOpen())
 
         services = {
             "sysvinit": {
@@ -499,13 +534,13 @@ class TestServicesHandler(testtools.TestCase):
                 mock_popen.assert_has_calls(calls)
             mock_exists.assert_called_with('/bin/systemctl')
 
-    def test_services_handler_sysv_disabled_systemctl(self):
+    def test_services_handler_sysv_disabled_systemctl(self, mock_cp):
         calls = []
 
         # apply_services
-        calls.append('/bin/systemctl disable httpd.service')
-        calls.append('/bin/systemctl status httpd.service')
-        calls.append('/bin/systemctl stop httpd.service')
+        calls.append(['/bin/systemctl', 'disable', 'httpd.service'])
+        calls.append(['/bin/systemctl', 'status', 'httpd.service'])
+        calls.append(['/bin/systemctl', 'stop', 'httpd.service'])
 
         calls = popen_root_calls(calls)
 
@@ -532,31 +567,35 @@ class TestServicesHandler(testtools.TestCase):
                 mock_popen.assert_has_calls(calls)
             mock_exists.assert_called_with('/bin/systemctl')
 
-    def test_services_handler_sysv_service_updaterc(self):
+    def test_services_handler_sysv_service_updaterc(self, mock_cp):
         calls = []
         returns = []
 
         # apply_services
-        calls.append('/usr/sbin/update-rc.d httpd enable')
+        calls.append(['/usr/sbin/update-rc.d', 'httpd', 'enable'])
         returns.append(FakePOpen())
-        calls.append('/usr/sbin/service httpd status')
+        calls.append(['/usr/sbin/service', 'httpd', 'status'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/usr/sbin/service httpd start')
+        calls.append(['/usr/sbin/service', 'httpd', 'start'])
         returns.append(FakePOpen())
 
         # monitor_services not running
-        calls.append('/usr/sbin/service httpd status')
+        calls.append(['/usr/sbin/service', 'httpd', 'status'])
         returns.append(FakePOpen(returncode=-1))
-        calls.append('/usr/sbin/service httpd start')
-        returns.append(FakePOpen())
-        calls.append('/bin/services_restarted')
+        calls.append(['/usr/sbin/service', 'httpd', 'start'])
         returns.append(FakePOpen())
 
-        # monitor_services running
-        calls.append('/usr/sbin/service httpd status')
+        shell_calls = []
+        shell_calls.append('/bin/services_restarted')
         returns.append(FakePOpen())
 
         calls = popen_root_calls(calls)
+        calls.extend(popen_root_calls(shell_calls, shell=True))
+
+        # monitor_services running
+        calls.extend(popen_root_calls([['/usr/sbin/service', 'httpd',
+                                        'status']]))
+        returns.append(FakePOpen())
 
         services = {
             "sysvinit": {
@@ -588,16 +627,16 @@ class TestServicesHandler(testtools.TestCase):
             mock_exists.assert_any_call('/sbin/service')
             mock_exists.assert_any_call('/sbin/chkconfig')
 
-    def test_services_handler_sysv_disabled_service_updaterc(self):
+    def test_services_handler_sysv_disabled_service_updaterc(self, mock_cp):
         calls = []
         returns = []
 
         # apply_services
-        calls.append('/usr/sbin/update-rc.d httpd disable')
+        calls.append(['/usr/sbin/update-rc.d', 'httpd', 'disable'])
         returns.append(FakePOpen())
-        calls.append('/usr/sbin/service httpd status')
+        calls.append(['/usr/sbin/service', 'httpd', 'status'])
         returns.append(FakePOpen())
-        calls.append('/usr/sbin/service httpd stop')
+        calls.append(['/usr/sbin/service', 'httpd', 'stop'])
         returns.append(FakePOpen())
 
         calls = popen_root_calls(calls)
@@ -659,7 +698,8 @@ interval=120''' % fcreds.name).encode('UTF-8'))
         self.assertIn('invalid credentials file', str(e))
         fcreds.close()
 
-    def test_hup_config(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_hup_config(self, mock_cp):
         hooks_conf = tempfile.NamedTemporaryFile()
 
         def write_hook_conf(f, name, triggers, path, action):
@@ -734,11 +774,11 @@ interval=120''' % fcreds.name).encode('UTF-8'))
             ' root, /bin/hook3}', str(hooks[3]))
 
         calls = []
-        calls.append('/bin/cfn-http-restarted')
-        calls.append('/bin/hook1')
-        calls.append('/bin/hook2')
-        calls.append('/bin/hook3')
-        calls = popen_root_calls(calls)
+        calls.extend(popen_root_calls(['/bin/cfn-http-restarted'], shell=True))
+        calls.extend(popen_root_calls(['/bin/hook1'], shell=True))
+        calls.extend(popen_root_calls(['/bin/hook2'], shell=True))
+        calls.extend(popen_root_calls(['/bin/hook3'], shell=True))
+        #calls = popen_root_calls(calls)
 
         with mock.patch('subprocess.Popen') as mock_popen:
             mock_popen.return_value = FakePOpen('All good')
@@ -1070,7 +1110,8 @@ class TestMetadataRetrieve(testtools.TestCase):
 
             self.assertEqual(meta_in, meta_out)
 
-    def test_nova_meta_curl(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_nova_meta_curl(self, mock_cp):
         url = 'http://169.254.169.254/openstack/2012-08-10/meta_data.json'
         temp_home = tempfile.mkdtemp()
         cache_path = os.path.join(temp_home, 'meta_data.json')
@@ -1103,9 +1144,10 @@ class TestMetadataRetrieve(testtools.TestCase):
             meta_out = md.get_nova_meta(cache_path=cache_path)
             self.assertEqual(meta_in, meta_out)
             mock_popen.assert_has_calls(
-                popen_root_calls(['curl -o %s %s' % (cache_path, url)]))
+                popen_root_calls([['curl', '-o', cache_path, url]]))
 
-    def test_nova_meta_curl_corrupt(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_nova_meta_curl_corrupt(self, mock_cp):
         url = 'http://169.254.169.254/openstack/2012-08-10/meta_data.json'
         temp_home = tempfile.mkdtemp()
         cache_path = os.path.join(temp_home, 'meta_data.json')
@@ -1131,9 +1173,10 @@ class TestMetadataRetrieve(testtools.TestCase):
             meta_out = md.get_nova_meta(cache_path=cache_path)
             self.assertEqual(None, meta_out)
             mock_popen.assert_has_calls(
-                popen_root_calls(['curl -o %s %s' % (cache_path, url)]))
+                popen_root_calls([['curl', '-o', cache_path, url]]))
 
-    def test_nova_meta_curl_failed(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_nova_meta_curl_failed(self, mock_cp):
         url = 'http://169.254.169.254/openstack/2012-08-10/meta_data.json'
         temp_home = tempfile.mkdtemp()
         cache_path = os.path.join(temp_home, 'meta_data.json')
@@ -1149,7 +1192,7 @@ class TestMetadataRetrieve(testtools.TestCase):
             meta_out = md.get_nova_meta(cache_path=cache_path)
             self.assertEqual(None, meta_out)
             mock_popen.assert_has_calls(
-                popen_root_calls(['curl -o %s %s' % (cache_path, url)]))
+                popen_root_calls([['curl', '-o', cache_path, url]]))
 
     def test_get_tags(self):
         fake_tags = {'foo': 'fee',
@@ -1209,7 +1252,8 @@ class TestCfnInit(testtools.TestCase):
             md.cfn_init()
             self.assertThat(foo_file.name, ttm.FileContains('bar'))
 
-    def test_cfn_init_with_ignore_errors_false(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_cfn_init_with_ignore_errors_false(self, mock_cp):
         md_data = {"AWS::CloudFormation::Init": {"config": {"commands": {
             "00_foo": {"command": "/bin/command1",
                        "ignoreErrors": "false"}}}}}
@@ -1219,16 +1263,18 @@ class TestCfnInit(testtools.TestCase):
             self.assertTrue(
                 md.retrieve(meta_str=md_data, last_path=self.last_file))
             self.assertRaises(cfn_helper.CommandsHandlerRunError, md.cfn_init)
-            mock_popen.assert_has_calls(popen_root_calls(['/bin/command1']))
+            mock_popen.assert_has_calls(popen_root_calls(['/bin/command1'],
+                                                         shell=True))
 
-    def test_cfn_init_with_ignore_errors_true(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_cfn_init_with_ignore_errors_true(self, mock_cp):
         calls = []
         returns = []
-        calls.append('/bin/command1')
+        calls.extend(popen_root_calls(['/bin/command1'], shell=True))
         returns.append(FakePOpen('Doing something', 'error', -1))
-        calls.append('/bin/command2')
+        calls.extend(popen_root_calls(['/bin/command2'], shell=True))
         returns.append(FakePOpen('All good'))
-        calls = popen_root_calls(calls)
+        #calls = popen_root_calls(calls)
 
         md_data = {"AWS::CloudFormation::Init": {"config": {"commands": {
             "00_foo": {"command": "/bin/command1",
@@ -1257,7 +1303,7 @@ class TestSourcesHandler(testtools.TestCase):
         sources = {dest: url}
         td = os.path.dirname(end_file)
         er = "mkdir -p '%s'; cd '%s'; curl -s '%s' | gunzip | tar -xvf -"
-        calls = popen_root_calls([er % (dest, dest, url)])
+        calls = popen_root_calls([er % (dest, dest, url)], shell=True)
 
         with mock.patch.object(tempfile, 'mkdtemp') as mock_mkdtemp:
             mock_mkdtemp.return_value = td
@@ -1268,26 +1314,28 @@ class TestSourcesHandler(testtools.TestCase):
                 mock_popen.assert_has_calls(calls)
             mock_mkdtemp.assert_called_with()
 
-    def test_apply_sources_github(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_apply_sources_github(self, mock_cp):
         url = "https://github.com/NoSuchProject/tarball/NoSuchTarball"
         dest = tempfile.mkdtemp()
         self.addCleanup(os.rmdir, dest)
         sources = {dest: url}
         er = "mkdir -p '%s'; cd '%s'; curl -s '%s' | gunzip | tar -xvf -"
-        calls = popen_root_calls([er % (dest, dest, url)])
+        calls = popen_root_calls([er % (dest, dest, url)], shell=True)
         with mock.patch('subprocess.Popen') as mock_popen:
             mock_popen.return_value = FakePOpen('Curl good')
             sh = cfn_helper.SourcesHandler(sources)
             sh.apply_sources()
             mock_popen.assert_has_calls(calls)
 
-    def test_apply_sources_general(self):
+    @mock.patch.object(cfn_helper, 'controlled_privileges')
+    def test_apply_sources_general(self, mock_cp):
         url = "https://website.no.existe/a/b/c/file.tar.gz"
         dest = tempfile.mkdtemp()
         self.addCleanup(os.rmdir, dest)
         sources = {dest: url}
         er = "mkdir -p '%s'; cd '%s'; curl -s '%s' | gunzip | tar -xvf -"
-        calls = popen_root_calls([er % (dest, dest, url)])
+        calls = popen_root_calls([er % (dest, dest, url)], shell=True)
         with mock.patch('subprocess.Popen') as mock_popen:
             mock_popen.return_value = FakePOpen('Curl good')
             sh = cfn_helper.SourcesHandler(sources)
